/***************************************************
 *                                                 *
 *   NWMap v. 1.1                                  *
 *   Copyright 2001, by Jason Harris               *
 *   http://nwmap.nwnonline.net/                   *
 *   nwmap@hotmail.com                             *
 *   jharris@30doradus.org                         *
 *                                                 *
 ***************************************************/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "NWDrawFuncs.h"
#include "NWMapMain.h"
#include "TilesetFormMain.h"
#include "DoorFormMain.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

void TForm1::drawMap() {
    Map->Canvas->Brush->Color = clWhite;
    Map->Canvas->FillRect(MBRect);

    int XTile1 = XTile0 + MapBox->Width / TileSize;
    int YTile1 = YTile0 + MapBox->Height / TileSize;

    for (int i=XTile0; i<XTile1; ++i) {
        for (int j=YTile0; j<YTile1; ++j) {
            drawTile(i,j);
        }
    }

    //Blit to screen
    MapBox->Canvas->Draw(0,0,Map);
}
//---------------------------------------------------------------------------

void TForm1::drawTile(int TileX, int TileY)
{
    int x = TileSize*(TileX - XTile0);// - Xoff;
    int y = TileSize*(TileY - YTile0);// - Yoff;
    MapRect = Rect(x, y, x+TileSize, y+TileSize);

    if (RectWithin(MapRect, MBRect)) {
        Tile->Canvas->Brush->Color = clDkGray;
        Tile->Canvas->FillRect( TileRect );
        for (int i=0; i<nmLayer; ++i) {
            if ( !OneLayerMode || i==CurrentLayer ) { //if in One Layer Mode, only draw current layer
                switch (i) {
                    case 0: // base layer (no transparency)
                        TilesetForm->getTile( Tile2, TileArray[TileX][TileY][0], RotArray[TileX][TileY][0], 0 );
                        Tile->Canvas->CopyMode = cmSrcCopy;
                        Tile->Canvas->Draw( 0, 0, Tile2 );
                        break;
                    default: //upper layers, including transition tiles in base layer (tlayer = mlayer - 1)
                        //Don't need to draw BlankTile's
                        if ((i==1 && TileArray[TileX][TileY][i] != NTILES - 1) ||
                            (i>=2 && TileArray[TileX][TileY][i] !=  0)) {
                                TilesetForm->getTile( Tile2, TileArray[TileX][TileY][i], RotArray[TileX][TileY][i], i-1 );
                                TransDraw( Tile, Tile2 );
                        }
                        break;
                }
            }
        }

        int iTile = TileY*NXMAP + TileX;

        if (WantLabels) {
            for (int i=0; i<TileList->Count; ++i) {
                NWTile *nwt = (NWTile *)TileList->Items[i];
                if (nwt->iTile == iTile) {
                    Tile->Canvas->Brush->Color = clWhite;
                    Tile->Canvas->Font->Color = clBlack;
                    int xc, yc;
                    if (Zoom==1.0) {
                        Tile->Canvas->Font->Height = TILESIZE;
                        xc = TILESIZE/4;
                        yc = 2;
                    } else if (Zoom==2.0) {
                        Tile->Canvas->Font->Height = TILESIZE/2;
                        xc = TILESIZE/3;
                        yc = TILESIZE/4;
                    } else {
                        Tile->Canvas->Font->Height = TILESIZE/4;
                        xc = TILESIZE/2;
                        yc = TILESIZE/4;
                    }
                    Tile->Canvas->TextOut(xc, yc, (String)(i+1));

                    break;  //we found the tile, stop looking
                }
            }
        }

        if (Zoom==1.0) {
            DoorForm->drawDoors(TileX, TileY, Tile->Canvas, 4);
        } else {
            DoorForm->drawDoors(TileX, TileY, Tile->Canvas);
        }

        Map->Canvas->CopyMode = cmSrcCopy;
        HDC mapDC = Map->Canvas->Handle;
        SetStretchBltMode( mapDC, COLORONCOLOR );
        Map->Canvas->CopyRect(MapRect, Tile->Canvas, TileRect);
        //Map->Canvas->StretchDraw(MapRect, Tile);
    }
}
//---------------------------------------------------------------------------
void TForm1::TransDraw( Graphics::TBitmap *base, Graphics::TBitmap *top )
{
    //Create stage-1 mask (magenta pixels are *transparent*)
    TileMask = new Graphics::TBitmap();
    TileMask->Width = TILESIZE;
    TileMask->Height = TILESIZE;
    TileMask->Canvas->CopyMode = cmSrcCopy;
    TileMask->Canvas->Draw( 0, 0, top );
    TileMask->Mask( TColor(0x00FF00FF) ); //make magenta transparent

    //Create stage-2 mask (magenta pixels are *white*)
    TileMask2->Canvas->Brush->Color = clWhite;
    TileMask2->Canvas->FillRect( TileRect );
    TileMask2->Canvas->CopyMode = cmSrcAnd;
    TileMask2->Canvas->Draw( 0, 0, TileMask );

    delete TileMask;

    //Create inverted Mask
    InvMask->Canvas->CopyMode = cmNotSrcCopy;
    InvMask->Canvas->Draw( 0, 0, TileMask2 );

    //AND-combine InvMask and Tile2 (making Magenta pixels black in new layer)
    top->Canvas->CopyMode = cmSrcAnd;
    top->Canvas->Draw( 0, 0, InvMask );

    //AND-combine TileMask2 and Tile (making non-Magenta pixels black in base layer(s))
    base->Canvas->CopyMode = cmSrcAnd;
    base->Canvas->Draw( 0, 0, TileMask2 );

    //OR-combine the base layer(s) and the new layer
    base->Canvas->CopyMode = cmSrcPaint;
    base->Canvas->Draw( 0, 0, top );
}

//---------------------------------------------------------------------------

void TForm1::drawCursor(int TileX, int TileY)
{
    int x = TileSize*(TileX - XTile0);
    int y = TileSize*(TileY - YTile0);
    MapRect = Rect(x, y, x+TileSize, y+TileSize);

    HDC tileDC = Tile->Canvas->Handle;
    SetStretchBltMode( tileDC, COLORONCOLOR );
    HDC tile2DC = Tile2->Canvas->Handle;
    SetStretchBltMode( tile2DC, COLORONCOLOR );
    HDC saveDC = SaveTile->Canvas->Handle;
    SetStretchBltMode( saveDC, COLORONCOLOR );
    HDC mapDC = Map->Canvas->Handle;
    SetStretchBltMode( mapDC, COLORONCOLOR );

    drawTile(CurrentTileX, CurrentTileY);
    SaveTile->Canvas->CopyMode = cmSrcCopy; //DEST = SRC
    SaveTile->Canvas->CopyRect(TileRect, Map->Canvas, MapRect);

    Graphics::TBitmap *PreviewTile = new Graphics::TBitmap();
    PreviewTile->Width = TILESIZE;
    PreviewTile->Height = TILESIZE;

    for (int i=0; i<nmLayer; ++i) {
        if (i==CurrentLayer) {// Draw Palette Tile (transparently weaved with map tile)
            TRect MaskRect = TileRect;
            if (Zoom==1.0) MaskRect = Rect(0,0,16,16); //Mask is sometimes visible at this zoom! Trying different MaskRect sizes...
            if (Zoom==2.0) MaskRect = Rect(0,0,40,40);

            Graphics::TBitmap *PTile = new Graphics::TBitmap();
            PTile->Width = TILESIZE;
            PTile->Height = TILESIZE;
            Graphics::TBitmap *QTile = new Graphics::TBitmap();
            QTile->Width = TILESIZE;
            QTile->Height = TILESIZE;

            TilesetForm->getTile(PTile, TilesetForm->SelectedTile[TilesetForm->CurrentLayer], TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer], TilesetForm->CurrentLayer);
            TilesetForm->getTile(QTile, TilesetForm->SelectedTile[TilesetForm->CurrentLayer], TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer], TilesetForm->CurrentLayer);
            TilesetForm->getTile(Tile2, TileArray[TileX][TileY][CurrentLayer], RotArray[TileX][TileY][CurrentLayer], TilesetForm->CurrentLayer);
            PTile->Canvas->CopyMode = cmSrcAnd; //DEST = DEST & SRC
            PTile->Canvas->CopyRect(TileRect, Mask->Canvas, MaskRect);

            Tile2->Canvas->CopyMode = cmSrcAnd; //DEST = DEST & SRC
            Tile2->Canvas->CopyRect(TileRect, Mask2->Canvas, MaskRect);
            Tile2->Canvas->CopyMode = cmSrcCopy; //DEST = SRC

            PTile->Canvas->CopyMode = cmSrcPaint; //DEST = SRC || DEST
            PTile->Canvas->CopyRect(TileRect, Tile2->Canvas, TileRect);
            PTile->Canvas->CopyMode = cmSrcCopy; //DEST = SRC

            if (i==0) { //base layer, no transparency
                Tile->Canvas->CopyMode = cmSrcCopy;
                Tile->Canvas->Draw( 0, 0, PTile );
                PreviewTile->Canvas->CopyMode = cmSrcCopy;
                PreviewTile->Canvas->Draw( 0, 0, QTile );
            } else {
                TransDraw( Tile, PTile );
                TransDraw( PreviewTile, QTile );
            }

            delete PTile;
            delete QTile;

        } else if ( !OneLayerMode ) { // Draw non-current layer transparently straight from the map
            if (i==0) { //base layer, no transparency
                TilesetForm->getTile( Tile2, TileArray[TileX][TileY][i], RotArray[TileX][TileY][i], i );
                Tile->Canvas->CopyMode = cmSrcCopy;
                Tile->Canvas->Draw( 0, 0, Tile2 );
                PreviewTile->Canvas->CopyMode = cmSrcCopy;
                PreviewTile->Canvas->Draw( 0, 0, Tile2 );
            } else {
                TilesetForm->getTile( Tile2, TileArray[TileX][TileY][i], RotArray[TileX][TileY][i], i-1 );
                TransDraw( Tile, Tile2 );
                TilesetForm->getTile( Tile2, TileArray[TileX][TileY][i], RotArray[TileX][TileY][i], i-1 );
                TransDraw( PreviewTile, Tile2 );
            }
        }
    }

    PreviewImage->Canvas->StretchDraw( Rect( 0, 0, PreviewImage->Width, PreviewImage->Height ), PreviewTile );
    delete PreviewTile;

    if (Zoom==1.0) {
        DoorForm->drawDoors(TileX, TileY, Tile->Canvas, 4);
    } else {
        DoorForm->drawDoors(TileX, TileY, Tile->Canvas);
    }

    Map->Canvas->CopyRect( MapRect, Tile->Canvas, TileRect );

    // Draw Tile Cursor around CurrentTile
    drawBox(x/TileSize, y/TileSize, x/TileSize+1, y/TileSize+1);
}
//---------------------------------------------------------------------------

void TForm1::drawLabels(int iStart, int iEnd) {
    if (WantLabels) {
        for (int i=iStart; i<iEnd; ++i) {
            NWTile *nwt = (NWTile *)TileList->Items[i];
            int iTile = nwt->iTile;
            int YTile = int(iTile/NXMAP);
            int XTile = iTile - YTile*NXMAP;

            //draw labels on map
            drawTile(XTile, YTile);
        }
        MapBox->Canvas->Draw(0,0,Map);
    }
}
//---------------------------------------------------------------------------
void TForm1::drawBox(int X1, int Y1, int X2, int Y2)
{
    Map->Canvas->MoveTo(TileSize*X1 + 2, TileSize*Y1 + 2);
    Map->Canvas->LineTo(TileSize*X2 - 2, TileSize*Y1 + 2);
    Map->Canvas->LineTo(TileSize*X2 - 2, TileSize*Y2 - 2);
    Map->Canvas->LineTo(TileSize*X1 + 2, TileSize*Y2 - 2);
    Map->Canvas->LineTo(TileSize*X1 + 2, TileSize*Y1 + 2);
}


bool TForm1::RectWithin(TRect r1, TRect r2) {
    //If any point of r1 lies within r2, return true
    if (r1.Left   >= r2.Left  && r1.Left   <= r2.Right  ||
        r1.Right  >= r2.Left  && r1.Right  <= r2.Right  ||
        r1.Top    >= r2.Top   && r1.Top    <= r2.Bottom ||
        r1.Bottom >= r2.Top   && r1.Bottom <= r2.Bottom) {
        return true;
    } else {
        return false;
    }
}

void TForm1::BlobDraw( void ) {
    //examList is the TList of tile coords that need to be examined for neighboring SelctedTile's.
    //Initialize examList to be the 8 neighbors of the current tile.
    TList *examList = new TList();

    for ( int ix=CurrentTileX-1; ix<=CurrentTileX+1; ++ix ) {
        for ( int iy=CurrentTileY-1; iy<=CurrentTileY+1; ++iy ) {
            //Add to examList each neighbor that is in the map and not already a SelectedTile

            if ( ix>=0 && ix<NXMAP && iy>=0 && iy<NYMAP &&
                TileArray[ix][iy][CurrentLayer]!=TilesetForm->SelectedTile[TilesetForm->CurrentLayer] ) {
                TPoint *p = new TPoint( ix, iy );
                examList->Add( p );
            }
        }
    }

    //Examine neigbors of each location in examList.  Because examList may change as
    //the examination proceeds, use a while loop to repeat the exam until there are
    //zero items left in examList.  For each iteration, examine the first item in the list,
    //and delete it when done.

    while ( examList->Count ) {
        TPoint *p = (TPoint *)examList->Items[0];
        int ix = p->x;  int iy = p->y;

        //The examination of neighbors has two stages:  First stage, check the 4
        //primary edge neighbors.  If no SelectedTiles are found, check the corners also.
        int nConfig = 0; //nConfig identifies which of the 16 possible configurations is present

        if (TileArray[ix][iy][CurrentLayer]!=TilesetForm->SelectedTile[TilesetForm->CurrentLayer]) {
            for ( int i=0; i<4; ++i ) { //loop through 4 primary neighbor positions
                int jx = ix + dx[i];  int jy = iy + dy[i];
                if ( jx >=0 && jx<NXMAP && jy>=0 && jy<NYMAP ) {
                    if ( TileArray[jx][jy][CurrentLayer] == TilesetForm->SelectedTile[TilesetForm->CurrentLayer] ) {
                        nConfig += val[i];
                    }
                }
            }
        } else { //abort count if the tile is now a SelectedTile.  Signal abort with nConfig=-1
            nConfig = -1;
        }

        //Set the current tile based on nConfig
        //not using switch() here because can't declare variables to have scope only inside a case
        if (nConfig==0) { //zero primary neighbors found, check corner neighbors
            int dcx[4] = { 1, -1, -1, 1 }; //each item in these arrays ID's a corner neighbor
            int dcy[4] = { 1, 1, -1, -1 };
            int cCount = 0;
            int cIndex = 0;

            for ( int ic=0; ic<4; ++ic ) { //Loop through corner neighbors
                int cx = ix + dcx[ic];  int cy = iy + dcy[ic];
                if ( cx >=0 && cx<NXMAP && cy>=0 && cy<NYMAP && TileArray[cx][cy][CurrentLayer] == TilesetForm->SelectedTile[TilesetForm->CurrentLayer] ) {
                    cCount++;
                    cIndex = ic;
                }
            }

            if ( cCount==1) { //one corner neighbor occupied; morph tile to corner of current transition group.
                TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 1; //Corner tile
                RotArray[ix][iy][CurrentLayer+1] = cIndex;
            } else if ( cCount > 1 ) { //Multiple corners are occupied; defer tile by sending it to the end of examList
                                        //(unless examList->Count==1)
                if (examList->Count > 1) {
                    TPoint *p = new TPoint( ix, iy );
                    examList->Add( p );
                }
            }

        } else if (nConfig==1) { //one neighbor found above
            TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 2; //Edge tile
            RotArray[ix][iy][CurrentLayer+1] = 2;

        } else if (nConfig==2) { //one neighbor found right
            TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 2; //Edge tile
            RotArray[ix][iy][CurrentLayer+1] = 3;

        } else if (nConfig==3) { //two neighbors, above and right
            TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 3; //Inside corner tile
            RotArray[ix][iy][CurrentLayer+1] = 3;

        } else if (nConfig==4) { //one neighbor, below
            TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 2; //Edge tile
            RotArray[ix][iy][CurrentLayer+1] = 0;

        } else if (nConfig==5) { //two neighbors, above and below
            //make this tile a SelectedTile, and add it to examList.
            TileArray[ix][iy][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            TileArray[ix][iy][CurrentLayer+1] = 99; //erase transition layer
            TPoint *p = new TPoint( ix, iy );
            examList->Add( p );

        } else if (nConfig==6) { //two neighbors below and right
            TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 3; //Inside corner tile
            RotArray[ix][iy][CurrentLayer+1] = 0;

        } else if (nConfig==7) { //three neighbors, above, below, right
            //make this tile a SelectedTile, and add it to examList.
            TileArray[ix][iy][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            TileArray[ix][iy][CurrentLayer+1] = 99; //erase transition layer
            TPoint *p = new TPoint( ix, iy );
            examList->Add( p );

        } else if (nConfig==8) { //one neighbor, left
            TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 2; //Edge tile
            RotArray[ix][iy][CurrentLayer+1] = 1;

        } else if (nConfig==9) { //two neighbors, above and left
            TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 3; //Inside corner tile
            RotArray[ix][iy][CurrentLayer+1] = 2;

        } else if (nConfig==10) { //two neighbors, left and right
            //make this tile a SelectedTile, and add it to examList.
            TileArray[ix][iy][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            TileArray[ix][iy][CurrentLayer+1] = 99; //erase transition layer
            TPoint *p = new TPoint( ix, iy );
            examList->Add( p );

        } else if (nConfig==11) { //three neighbors, above, left, right
            //make this tile a SelectedTile, and add it to examList.
            TileArray[ix][iy][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            TileArray[ix][iy][CurrentLayer+1] = 99; //erase transition layer
            TPoint *p = new TPoint( ix, iy );
            examList->Add( p );

        } else if (nConfig==12) { //two neighbors, left and below
            TileArray[ix][iy][CurrentLayer+1] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer] + 3; //Inside corner tile
            RotArray[ix][iy][CurrentLayer+1] = 1;

        } else if (nConfig==13) { //three neighbors, left, above, below
            //make this tile a SelectedTile, and add it to examList.
            TileArray[ix][iy][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            TileArray[ix][iy][CurrentLayer+1] = 99; //erase transition layer
            TPoint *p = new TPoint( ix, iy );
            examList->Add( p );

        } else if (nConfig==14) { //three neighbors, left, right, below
            //make this tile a SelectedTile, and add it to examList.
            TileArray[ix][iy][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            TileArray[ix][iy][CurrentLayer+1] = 99; //erase transition layer
            TPoint *p = new TPoint( ix, iy );
            examList->Add( p );

        } else if (nConfig==15) { //four neighbors
            //make this tile a SelectedTile, and add it to examList.
            TileArray[ix][iy][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            TileArray[ix][iy][CurrentLayer+1] = 99; //erase transition layer
            TPoint *p = new TPoint( ix, iy );
            examList->Add( p );
        }


        //Draw the Tile
        if (nConfig>-1) drawTile( ix, iy );

        //Remove this item from examList
        delete examList->Items[0];
        examList->Delete(0);

    } //This loop will end when examList->Count==0

    delete examList;
}

void TForm1::SnakeDraw( void ) {
    bool FoundInGroup;
    bool goMorph = true;
    int imode = -1*TilesetForm->gType[CurrentLayer-2][CurrentDrawMode-2] - 1;
    int k,k0;

    //Should the target tile be intersected?
    for (k=0; k<4; ++k) {
        //First: is the SCrossTile > 100?  if so, int(x/100) is the layer
        int tCross = TilesetForm->SCrossTile[CurrentLayer-2][imode][k];
        int tLayer = CurrentLayer;
        if (tCross > 100) {
            tLayer = int(tCross/100);
            tCross = tCross - 100*tLayer;
        }

        if ( TileArray[CurrentTileX][CurrentTileY][tLayer] == tCross ) {
            //First, set the SCrossTile tile in tLayer to the BlankTile.
            if (tLayer != CurrentLayer) {
                if (tLayer==1) {
                    TileArray[CurrentTileX][CurrentTileY][tLayer] = NTILES - 1;
                } else {
                    TileArray[CurrentTileX][CurrentTileY][tLayer] = 0;
                }
            }

            //Set the Tile in CurrentLayer to the SCReplaceTile.
            TileArray[CurrentTileX][CurrentTileY][CurrentLayer] = TilesetForm->SCReplaceTile[CurrentLayer-2][imode][k];
            RotArray[CurrentTileX][CurrentTileY][CurrentLayer] += TilesetForm->SCReplaceRot[CurrentLayer-2][imode][k];

            while ( RotArray[CurrentTileX][CurrentTileY][CurrentLayer] > 3 )
                RotArray[CurrentTileX][CurrentTileY][CurrentLayer] -= 4;

            goMorph = false; // Intersected tile should not be morphed
            break;
        }
    }

    //Don't morph the tile if it is already intersected, or already an end tile.
    for (k=0; k<4; ++k) {
        if ( TileArray[CurrentTileX][CurrentTileY][CurrentLayer]==TilesetForm->SCReplaceTile[CurrentLayer-2][imode][k] ||
             TileArray[CurrentTileX][CurrentTileY][CurrentLayer]==TilesetForm->SEndTile[CurrentLayer-2][imode][k] ) {
            goMorph = false;
            break;
        }
    }

    //Should the target tile be an End Tile?
    for ( int k=0; k<4; ++k ) { //check all 4 possible end-flag tiles
        //First: is the EndFlag > 100?  if so, int(x/100) is the layer
        int tEnd = TilesetForm->SEndFlag[CurrentLayer-2][imode][k];
        int tLayer = CurrentLayer;
        if ( tEnd > 100 ) {
            tLayer = int(tEnd/100);
            tEnd = tEnd - 100*tLayer;
        }

        if ( TileArray[CurrentTileX][CurrentTileY][tLayer] == tEnd ) {
            //First, set the SEndFlag tile in tLayer to the BlankTile.
            if (tLayer==1) {
                TileArray[CurrentTileX][CurrentTileY][tLayer] = NTILES - 1;
            } else {
                TileArray[CurrentTileX][CurrentTileY][tLayer] = 0;
            }

            //Set the Tile in CurrentLayer to the SEndTile.
            TileArray[CurrentTileX][CurrentTileY][CurrentLayer] = TilesetForm->SEndTile[CurrentLayer-2][imode][k];
            RotArray[CurrentTileX][CurrentTileY][CurrentLayer] = RotArray[CurrentTileX][CurrentTileY][tLayer] + TilesetForm->SEndRot[CurrentLayer-2][imode][k];

            while ( RotArray[CurrentTileX][CurrentTileY][CurrentLayer] > 3 )
                RotArray[CurrentTileX][CurrentTileY][CurrentLayer] -= 4;

            goMorph = false; // Intersected tile should not be morphed
            break;

        }
    }

    //Place current tile so that it merges nicely with its neighbors
    if (goMorph) TileMorph(CurrentTileX, CurrentTileY);

    //Loop through neighbors, and update those that are in the current snakegroup.
    for (int i=0; i<4; ++i) { // Loop throught 4 neighboring tiles of CurrentTile
        int xt = CurrentTileX + dx[i]; //dx = { 0, 1, 0, -1 }
        int yt = CurrentTileY + dy[i]; //dy = { -1, 0, 1, 0 }
        FoundInGroup = false;

        if (xt>=0 && xt<NXMAP && yt>=0 && yt<NYMAP) {

            for (int k=0; k<4; ++k) {
                if (TileArray[xt][yt][CurrentLayer]==TilesetForm->STile[CurrentLayer-2][imode][k] ) {
                    FoundInGroup = true;
                    break;
                }
            }
            if (FoundInGroup) TileMorph(xt,yt);
        }
    }
}

void TForm1::TileMorph( int x, int y ) {
    int imode = -1*TilesetForm->gType[CurrentLayer-2][CurrentDrawMode-2] - 1;

//    if (goMorph) {
        int index = 0;
        /*
        index is a bitmap variable that identifies which of the 16 possible neighbor
        configurations the current tile is experiencing:
        dx: { 0, 1, 0, -1 }
        dy: { -1, 0, 1, 0 }
        so, looping through i and adding dx, dy to current coords loops
        through directions: above, right, below, left

        val: { 1, 2, 4, 8}
        so, if the neighbor below the target tile is in the group, index is incremented by 1
        if the neighbor right of the target tile is in the group, index is incremented by 2
        if the neighbor above the target tile is in the group, index is incremented by 4
        if the neighbor left of the target tile is in the group, index is incremented by 8
        After all 4 neighbors are checked, index is a number between 0 and 15 that uniquely identifies
        the configuration of neighboring tiles that are in the group.
        */

        for (int i=0; i<4; ++i) { // loop through 4 positions: above, right, below, left
            int xt = x + dx[i];
            int yt = y + dy[i];

            if (xt>=0 && xt<NXMAP && yt>=0 && yt<NYMAP) {
                for (int j=0; j<4; ++j) {
                    if (TileArray[xt][yt][CurrentLayer]==TilesetForm->STile[CurrentLayer-2][imode][j] || // is nbr tile in the group?
                        TileArray[xt][yt][CurrentLayer]==TilesetForm->SCReplaceTile[CurrentLayer-2][imode][j] ||
                        TileArray[xt][yt][CurrentLayer]==TilesetForm->SEndTile[CurrentLayer-2][imode][j]) {
                        index += val[i]; // accumulate the index
                    }
                }
            }
        }

        switch (index) {
            case 0: // no neigbors
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][0]; // straight, horizontal segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][0];
                break;
            case 1: // 1 neigbor above
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][0]; // straight, vertical segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][0] + 1;
                break;
            case 2: // 1 neigbor right
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][0]; // straight, horizontal segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][0];
                break;
            case 3: // 2 neigbors above and right
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][1]; // corner segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][1] + 3;
                break;
            case 4: // 1 neigbor below
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][0]; // straight, horizontal segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][0]+1;
                break;
            case 5: // 2 neigbors above and below
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][0]; // straight, vertical segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][0] + 1;
                break;
            case 6: // 2 neigbors below and right
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][1]; // corner segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][1];
                break;
            case 7: // 3 neigbors above, below and right
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][2]; // "T" segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][2] + 3;
                break;
            case 8: // 1 neigbor left
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][0]; // straight, horizontal segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][0];
                break;
            case 9: // 2 neigbors above and left
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][1]; // corner segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][1] + 2;
                break;
            case 10: // 2 neigbors left and right
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][0]; // straight, horizontal segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][0];
                break;
            case 11: // 3 neigbors left, right and above
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][2]; // "T" segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][2] + 2;
                break;
            case 12: // 2 neigbors below and left
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][1]; // corner segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][1] + 1;
                break;
            case 13: // 3 neigbors above, below and left
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][2]; // "T" segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][2] + 1;
                break;
            case 14: // 3 neigbors left, right and below
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][2]; // "T" segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][2];
                break;
            case 15: // all 4 neigbors
                TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][3]; // cross segment
                RotArray[x][y][CurrentLayer] = TilesetForm->SRot[CurrentLayer-2][imode][3];
                //For rivers, the cross segment is undefined (set to -1).  Check for this,
                //and reset to the "T" tile.
                if ( TileArray[x][y][CurrentLayer] == -1) {
                    TileArray[x][y][CurrentLayer] = TilesetForm->STile[CurrentLayer-2][imode][2];
                }
                break;
        }
//    }
    while (RotArray[x][y][CurrentLayer] >3) RotArray[x][y][CurrentLayer] -= 4;
    while (RotArray[x][y][CurrentLayer] <0) RotArray[x][y][CurrentLayer] += 4;

    drawTile(x,y);
}

