/***************************************************
 *                                                 *
 *   NWMap v. 1.1                                  *
 *   Copyright 2001, by Jason Harris               *
 *   http://nwmap.nwnonline.net/                   *
 *   nwmap@hotmail.com                             *
 *   jharris@30doradus.org                         *
 *                                                 *
 ***************************************************/
//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdlib.h>
#pragma hdrstop

#include "NWKeyFuncs.h"
#include "NWMapMain.h"
#include "TilesetFormMain.h"
#include "DoorFormMain.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

/*****************Block:  Keyboard Controls*************/
//KeyUp is used for non-alphanumeric keys (not representable by a char)
void __fastcall TForm1::FormKeyUp(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    bool DoorsChanged = false;

    switch (Key) {
        case VK_UP : //Scroll up
            if (Zoom > 1.0) YScroll->Position -= YScroll->SmallChange;
            break;
        case VK_DOWN : //Scroll down
            if (Zoom > 1.0) YScroll->Position += YScroll->SmallChange;
            break;
        case VK_LEFT : //Scroll left
            if (Zoom > 1.0) XScroll->Position -= XScroll->SmallChange;
            break;
        case VK_RIGHT : //Scroll right
            if (Zoom > 1.0) XScroll->Position += XScroll->SmallChange;
            break;

        case VK_NUMPAD8 : //North Door
            if (Doors[CurrentTileX][CurrentTileY][0]) { Doors[CurrentTileX][CurrentTileY][0] = 0; }
            else { Doors[CurrentTileX][CurrentTileY][0] = 1; }
            DoorsChanged = true;
            break;
        case VK_NUMPAD6 : //East Door
            if (Doors[CurrentTileX][CurrentTileY][1]) { Doors[CurrentTileX][CurrentTileY][1] = 0; }
            else { Doors[CurrentTileX][CurrentTileY][1] = 1; }
            DoorsChanged = true;
            break;
        case VK_NUMPAD2 : //South Door
            if (Doors[CurrentTileX][CurrentTileY][2]) { Doors[CurrentTileX][CurrentTileY][2] = 0; }
            else { Doors[CurrentTileX][CurrentTileY][2] = 1; }
            DoorsChanged = true;
            break;
        case VK_NUMPAD4 : //West Door
            if (Doors[CurrentTileX][CurrentTileY][3]) { Doors[CurrentTileX][CurrentTileY][3] = 0; }
            else { Doors[CurrentTileX][CurrentTileY][3] = 1; }
            DoorsChanged = true;
            break;
        case VK_NUMPAD5 : //Center Door (2 choices-- horiz/vert)
            if (Doors[CurrentTileX][CurrentTileY][4]==0) { Doors[CurrentTileX][CurrentTileY][4] = 1; }
            else if (Doors[CurrentTileX][CurrentTileY][4]==1) { Doors[CurrentTileX][CurrentTileY][4] = -1; }
            else { Doors[CurrentTileX][CurrentTileY][4] = 0; }
            DoorsChanged = true;
            break;
        case 90: //z
            if (Shift.Contains(ssCtrl)) {
                if (UndoButton->Enabled) UndoButtonClick( Form1 );
            }
            break;
        case 82: //r
            if (Shift.Contains(ssCtrl)) {
                if (RedoButton->Enabled) RedoButtonClick( Form1 );
            }
            break;
        case 88: //x
            if (Shift.Contains(ssCtrl)) {
                if (Shift.Contains(ssAlt)) {
                    if (CutDesc->Enabled) CutDescClick( Form1 );
                } else if (CutRegion->Enabled) {
                    CutRegionClick( CutRegion );
                }
            }
            break;
        case 67: //c
            if (Shift.Contains(ssCtrl)) {
                if (Shift.Contains(ssAlt)) {
                    if (CopyDesc->Enabled) CopyDescClick( Form1 );
                } else if (CopyRegion->Enabled) {
                    CutRegionClick( CopyRegion );
                }
            }
            break;
        case 86: //v
            if (Shift.Contains(ssCtrl)) {
                if (Shift.Contains(ssAlt)) {
                    if (PasteDesc->Enabled) PasteDescClick( Form1 );
                } else if (PasteRegion->Enabled) {
                    PasteRegionClick( PasteRegion );
                }
            }
            break;
    }

    if (DoorsChanged) {
        SaveMapState(true);
        drawCursor(CurrentTileX, CurrentTileY);
        if (Zoom==1.0) {
            DoorForm->drawDoors(CurrentTileX, CurrentTileY, SaveTile->Canvas, 4);
        } else {
            DoorForm->drawDoors(CurrentTileX, CurrentTileY, SaveTile->Canvas);
        }
        MapBox->Canvas->Draw(0,0,Map);
        MapChanged = true;
        SaveButton->Enabled = true;
    }


}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormKeyPress(TObject *Sender, char &Key)
{
    bool ZoomChange = false;
    bool RotateChange = false;

    switch (Key) {
        case ' ' : // Select CurrentTile
            // first, redraw old tile without cursor
            TilesetForm->drawTile(TilesetForm->SelectedTile[TilesetForm->CurrentLayer], TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer], TilesetForm->CurrentLayer);
            // select new tile in palette from map selection, match the rotation
            TilesetForm->SelectedTile[TilesetForm->CurrentLayer] = TileArray[CurrentTileX][CurrentTileY][CurrentLayer];
            TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer] = RotArray[CurrentTileX][CurrentTileY][CurrentLayer];
            TilesetForm->drawCursor(TilesetForm->SelectedTile[TilesetForm->CurrentLayer], TilesetForm->CurrentLayer, c);
            TilesetForm->TileBox[TilesetForm->CurrentLayer]->Canvas->Draw(0,0,TilesetForm->Tileset[TilesetForm->CurrentLayer]);
            //Enter draw by tile mode:
            CurrentDrawMode = 0;
            SnakeMode = false;
            Label1->Caption = DrawModes[TilesetForm->CurrentLayer]->Strings[CurrentDrawMode];
            //Draw new cursor:
            drawCursor(CurrentTileX, CurrentTileY);
            MapBox->Canvas->Draw(0,0,Map);
            break;
        case '-' : // Zoom out
        case '_' :
            Zoom *= 0.5;
            ZoomChange = true;
            break;
        case '=' : // Zoom in
        case '+' :
            Zoom *= 2.0;
            ZoomChange = true;
            break;
        case ',' : // Rotate Tile left
        case '<' :
            --TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer];
            if (TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer] < 0) TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer] = 3;
            RotateChange = true;
            break;
        case '.' : // Rotate Tile right
        case '>' :
            ++TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer];
            if (TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer] > 3) TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer] = 0;
            RotateChange = true;
            break;
        case 'h' : // toggle visibility of palette window
        case 'H' :
            TilesetForm->Visible = !TilesetForm->Visible;
            break;
        case 't' :
        case 'T' :
            DescItemClick(Form1);
            break;
        case 'd' :
        case 'D' :
            DoorItemClick(Form1);
            break;
        case 's' :
        case 'S' :
            if (TilesetForm->SizeFactor == 2) {
                TilesetForm->SizeFactor = 1;
                TilesetSizeItem->Checked = false;

                TilesetForm->CheckBox1->Top = 4;
                TilesetForm->CheckBox2->Top = 24;
                TilesetForm->CheckBox3->Top = 44;
                TilesetForm->CheckBox4->Top = 64;
                TilesetForm->CheckBox5->Top = 84;
                TilesetForm->CheckBox6->Top = 104;
                TilesetForm->CheckBox7->Top = 124;
                TilesetForm->CheckBox8->Top = 144;
                TilesetForm->CheckBox9->Top = 164;
                TilesetForm->CheckBox10->Top = 184;
                TilesetForm->Panel3->Height = 200;

                for (int i=0; i<TLAYER-1; ++i) {
                    TilesetForm->Panel[i][2]->Left = 40;
                    TilesetForm->Panel[i][2]->Top = 0;
                }

                TilesetForm->TileTitle->Width = 100;
                TilesetForm->oneLayerBox->Left = 110;
                TilesetForm->oneLayerBox->Width = 70;
                TilesetForm->OnTopBox->Left = 180;

                for (int i=1; i<TLAYER; ++i) {
                    TilesetForm->TileBox[i]->Left = 80;
                }

            } else {
                TilesetForm->SizeFactor = 2;
                TilesetSizeItem->Checked = true;

                TilesetForm->CheckBox1->Top = 16;
                TilesetForm->CheckBox2->Top = 56;
                TilesetForm->CheckBox3->Top = 96;
                TilesetForm->CheckBox4->Top = 136;
                TilesetForm->CheckBox5->Top = 176;
                TilesetForm->CheckBox6->Top = 216;
                TilesetForm->CheckBox7->Top = 256;
                TilesetForm->CheckBox8->Top = 296;
                TilesetForm->CheckBox9->Top = 336;
                TilesetForm->CheckBox10->Top = 376;
                TilesetForm->Panel3->Height = 402;

                for (int i=0; i<TLAYER-1; ++i) {
                    TilesetForm->Panel[i][2]->Left = 0;
                    TilesetForm->Panel[i][2]->Top = 200;
                }

                TilesetForm->TileTitle->Width = 200;
                TilesetForm->oneLayerBox->Left = 210;
                TilesetForm->oneLayerBox->Width = 120;
                TilesetForm->OnTopBox->Left = 350;

                for (int i=1; i<TLAYER; ++i) {
                    TilesetForm->TileBox[i]->Left = 40;
                }

            }

            TilesetForm->FormShow(Form1);
            break;

    }

    if (RotateChange) {
        TilesetForm->drawTile(TilesetForm->SelectedTile[TilesetForm->CurrentLayer], TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer], TilesetForm->CurrentLayer);
        TilesetForm->drawCursor(TilesetForm->SelectedTile[TilesetForm->CurrentLayer], TilesetForm->CurrentLayer, c);
        TilesetForm->TileBox[TilesetForm->CurrentLayer]->Canvas->Draw(0,0,TilesetForm->Tileset[TilesetForm->CurrentLayer]);
        drawCursor(CurrentTileX, CurrentTileY);
        MapBox->Canvas->Draw(0,0,Map);
    }

    if (ZoomChange) {
        ZoomInButton->Enabled = true;
        ZoomOutButton->Enabled = true;
        if (Zoom <= MINZOOM) { Zoom = (float)MINZOOM; ZoomOutButton->Enabled = false; }
        if (Zoom >= MAXZOOM) { Zoom = (float)MAXZOOM; ZoomInButton->Enabled = false; }

        TileSize = int(ROOTSIZE*Zoom);
        ScrollStep = int(TileSize/SCRLUNIT);
        XScroll->Max = max(NSCRL - MapBox->Width/ScrollStep, 0);
        YScroll->Max = max(NSCRL - MapBox->Height/ScrollStep, 0);

        if (NXMAP*TileSize > MapBox->Width) {
            XScroll->Visible = true;
            XScroll->Enabled = true;
        } else {
            XScroll->Visible = false;
            XScroll->Enabled = false;
        }

        if (NYMAP*TileSize > MapBox->Height) {
            YScroll->Visible = true;
            YScroll->Enabled = true;
        } else {
            YScroll->Visible = false;
            YScroll->Enabled = false;
        }

        drawMap();
    }
}
