/***************************************************
 *                                                 *
 *   NWMap v. 1.1                                  *
 *   Copyright 2001, by Jason Harris               *
 *   http://nwmap.nwnonline.net/                   *
 *   nwmap@hotmail.com                             *
 *   jharris@30doradus.org                         *
 *                                                 *
 ***************************************************/
//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdlib.h>
#pragma hdrstop

#include "NWMouseFuncs.h"
#include "NWMapMain.h"
#include "TilesetFormMain.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

void __fastcall TForm1::MapBoxMouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y)
{
    HDC mapDC = Map->Canvas->Handle;
    SetStretchBltMode( mapDC, COLORONCOLOR );

    CurrentTileX = (X/TileSize) + XTile0;
    CurrentTileY = (Y/TileSize) + YTile0;
    CurrentTile  = CurrentTileY*NXMAP + CurrentTileX;

    if (CurrentTileX >=0 && CurrentTileX <NXMAP && CurrentTileY >=0 && CurrentTileY < NYMAP) {
        if (CurrentTileX != LastTileX || CurrentTileY != LastTileY) {
            if (LastTileX>=0) { // LastTile = -1 on startup; don't redraw in this case
                // Redraw The previously occupied tile without the red tile cursor
                int x = TileSize*(LastTileX - XTile0);
                int y = TileSize*(LastTileY - YTile0);
                MapRect = Rect(x, y, x+TileSize, y+TileSize);

                Map->Canvas->CopyMode = cmSrcCopy;
                Map->Canvas->CopyRect(MapRect, SaveTile->Canvas, TileRect);

                //If we erased part of the selected box, redraw it
                if (!MouseDown && RegionSelected) {
                    if ((LastTileX==RectX1 || LastTileX==RectX2-1) ||
                        (LastTileY==RectY1 || LastTileY==RectY2-1)) {
                        drawBox(RectX1, RectY1, RectX2, RectY2);
                    }
                }
            }

            if (MouseDown) {
                if (CurrentDrawMode==0) {
                    TileArray[CurrentTileX][CurrentTileY][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
                    RotArray[CurrentTileX][CurrentTileY][CurrentLayer] = TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer];
                    drawTile(CurrentTileX, CurrentTileY);
                } else if (SnakeMode==true) {
                    SnakeDraw();
                } else if (CurrentDrawMode<0) {
                    TileArray[CurrentTileX][CurrentTileY][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
                    RotArray[CurrentTileX][CurrentTileY][CurrentLayer] = 0;
                    TileArray[CurrentTileX][CurrentTileY][CurrentLayer+1] = 99; //Clear transition layer
                    RotArray[CurrentTileX][CurrentTileY][CurrentLayer+1] = 0;
                    BlobDraw();
                }
            }

            LastTileX = CurrentTileX;
            LastTileY = CurrentTileY;

            if (MouseDown && CurrentDrawMode > 0 && SnakeMode == false) {
                //Erase Last Selection Rectangle
                X1 = min(X1,CurrentTileX);
                X2 = max(X2,CurrentTileX);
                Y1 = min(Y1,CurrentTileY);
                Y2 = max(Y2,CurrentTileY);

                for (int ix=X1-1; ix<=X2; ++ix) {
                    for (int iy=Y1-1; iy<=Y2; ++iy) {
                        if (ix>=0 && ix<=NXMAP && iy>=0 && iy<=NYMAP) {
                            drawTile(ix, iy);
                        }
                    }
                }

                //Draw New Selection Rectangle
                int XX1, XX2, YY1, YY2, t;
                XX1 = CurrentTileX-XTile0;
                XX2 = RectX1-XTile0;
                YY1 = CurrentTileY-YTile0;
                YY2 = RectY1-YTile0;

                if (XX1 > XX2) { t = XX1; XX1 = XX2; XX2 = t; }
                if (YY1 > YY2) { t = YY1; YY1 = YY2; YY2 = t; }
                if (XX1==XX2) { XX2 += 2; } else { ++XX2; }
                if (YY1==YY2) { YY2 += 2; } else { ++YY2; }

                drawBox(XX1, YY1, XX2, YY2);
            }

            drawCursor(CurrentTileX, CurrentTileY);
            MapBox->Canvas->Draw(0,0,Map);

            XTile->Caption = CurrentTileX + 1;
            YTile->Caption = CurrentTileY + 1;
        }
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MapBoxMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    CurrentTileX = (X/TileSize) + XTile0;
    CurrentTileY = (Y/TileSize) + YTile0;
    CurrentTile  = CurrentTileY*NXMAP + CurrentTileX;

    if (Button==mbLeft && !Shift.Contains(ssShift)) {
        MouseDown = true;

        if (CurrentDrawMode==0) {
            SaveMapState(true);
            TileArray[CurrentTileX][CurrentTileY][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            RotArray[CurrentTileX][CurrentTileY][CurrentLayer]  = TilesetForm->Rotation[TilesetForm->SelectedTile[TilesetForm->CurrentLayer]][TilesetForm->CurrentLayer];

            drawTile(CurrentTileX, CurrentTileY);
            drawCursor(CurrentTileX, CurrentTileY);
            MapBox->Canvas->Draw(0,0,Map);

        } else if (SnakeMode==true) {
            SaveMapState(true);
            SnakeDraw();

        } else if (CurrentDrawMode < 0) { //Transition layer mode
            SaveMapState(true);
            TileArray[CurrentTileX][CurrentTileY][CurrentLayer] = TilesetForm->SelectedTile[TilesetForm->CurrentLayer];
            RotArray[CurrentTileX][CurrentTileY][CurrentLayer] = 0;
            TileArray[CurrentTileX][CurrentTileY][CurrentLayer+1] = 99; //Clear transition layer
            RotArray[CurrentTileX][CurrentTileY][CurrentLayer+1] = 0;
            BlobDraw(); //Draw transition tiles in blob mode

        } else {
            if (CurrentDrawMode > 1) SaveMapState(true);  //Don't save state in selection mode
            RectX1 = CurrentTileX;
            RectY1 = CurrentTileY;
            X1 = NXMAP;
            X2 = XTile0;
            Y1 = NYMAP;
            Y2 = YTile0;
        }
    } else if (Button==mbRight && !Shift.Contains(ssShift)) {
        // Check current Tile in description list.  If it's there, enable cut/copy description popup menu items
        CutDesc->Enabled = false;
        CopyDesc->Enabled = false;

        for (int i=0; i<TileList->Count; ++i) {
            NWTile *nwt = (NWTile *)TileList->Items[i];
            if (nwt->iTile == CurrentTile) {
                CutDesc->Enabled = true;
                CopyDesc->Enabled = true;
                CurrentTileIndex = i;
                break; //found tile, stop looking
            }
        }
        MapMenu->Popup(Form1->Left + MapBox->Left + X,
                       Form1->Top  + MapBox->Top  + 36 + Y);
    }

}
//---------------------------------------------------------------------------

void __fastcall TForm1::MapBoxMouseUp(TObject *Sender, TMouseButton Button,
      TShiftState Shift, int X, int Y)
{

    if (Shift.Contains(ssShift)) {
        if (Button==mbLeft) {
            FormKeyPress(Form1, '<');
        } else if (Button==mbRight) {
            FormKeyPress(Form1, '>');
        }
    }
    MouseDown = false;

    if (SnakeMode) {
        int x = TileSize*(CurrentTileX - XTile0);
        int y = TileSize*(CurrentTileY - YTile0);
        MapRect = Rect(x, y, x+TileSize, y+TileSize);

        drawCursor(CurrentTileX, CurrentTileY);
        MapBox->Canvas->Draw(0,0,Map);
        MapChanged = true;
        SaveButton->Enabled = true;

    } else if ( CurrentDrawMode == 0 ) { // Normal mode
        MapChanged = true;
        SaveButton->Enabled = true;

    } else if ( CurrentDrawMode < 0 ) { //transition blob mode
        drawCursor(CurrentTileX, CurrentTileY);
        MapBox->Canvas->Draw(0,0,Map);
        MapChanged = true;
        SaveButton->Enabled = true;

    } else if ( CurrentDrawMode > 0 ) { //Select mode and Box modes
        int index, drot;
        RectX2 = CurrentTileX;
        RectY2 = CurrentTileY;

        if (RectX2 < RectX1) {
            int temp = RectX1;
            RectX1 = RectX2;
            RectX2 = temp;
        }
        if (RectY2 < RectY1) {
            int temp = RectY1;
            RectY1 = RectY2;
            RectY2 = temp;
        }

        if (RectX2==RectX1) ++RectX2;
        if (RectY2==RectY1) ++RectY2;

        if (CurrentDrawMode != 1) { //Box mode
            if (RectX1<0) RectX1 = 0;
            if (RectY1<0) RectY1 = 0;
            if (RectX2>NXMAP-1) RectX2 = NXMAP-1;
            if (RectY2>NYMAP-1) RectY2 = NYMAP-1;

            for (int ix=RectX1; ix<=RectX2; ++ix) {
                for (int iy=RectY1; iy<=RectY2; ++iy) {
                    if (iy==RectY1) {
                        if (ix==RectX1) { index = 1; drot = 0; } //Top left corner
                        else if (ix==RectX2) { index = 1; drot = 1; } //Top right corner
                        else { index = 2; drot = 0; } //Top edge
                    } else if (iy==RectY2) {
                        if (ix==RectX1) { index = 1; drot = 3; } //Bottom left corner
                        else if (ix==RectX2) { index = 1; drot = 2; } //Bottom right corner
                        else { index = 2; drot = 2; } //Bottom edge
                    } else {
                        if (ix==RectX1) { index = 2; drot = 3; } //Left edge
                        else if (ix==RectX2) { index = 2; drot = 1; } //Right edge
                        else { index = 0; } //Interior
                    }

                    if ( CurrentDrawMode > 1 ) { //user-define box modes
                        int imode = TilesetForm->gType[CurrentLayer-2][CurrentDrawMode-2] - 1;
                        //don't draw interior tiles if interior tile is BlankTile
                        if ( index || TilesetForm->BTile[CurrentLayer-2][imode][index] ) {

                            TileArray[ix][iy][CurrentLayer] = TilesetForm->BTile[CurrentLayer-2][imode][index];
                            RotArray[ix][iy][CurrentLayer]  = TilesetForm->BRot[CurrentLayer-2][imode][index] + drot;
                            if (RotArray[ix][iy][CurrentLayer] > 3) RotArray[ix][iy][CurrentLayer] -= 4;
                            drawTile( ix, iy );
                        }
                    }
                }
            }

            MapBox->Canvas->Draw(0,0,Map);
            MapChanged = true;
            SaveButton->Enabled = true;

        }

        if (CurrentDrawMode==1) { //Select Mode
            //Draw Select Rectangle
            ++RectX2;
            ++RectY2;
            drawBox(RectX1, RectY1, RectX2, RectY2);
            MapBox->Canvas->Draw(0,0,Map);
            RegionSelected = true;
            CutRegion->Enabled = true;
            CopyRegion->Enabled = true;
        }

        int x = TileSize*(CurrentTileX - XTile0);
        int y = TileSize*(CurrentTileY - YTile0);
        MapRect = Rect(x, y, x+TileSize, y+TileSize);

        HDC saveDC = SaveTile->Canvas->Handle;
        SetStretchBltMode( saveDC, COLORONCOLOR );

        SaveTile->Canvas->CopyMode = cmSrcCopy;
        SaveTile->Canvas->CopyRect(TileRect, Map->Canvas, MapRect);
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MapBoxDblClick(TObject *Sender)
{
    //Erase transition layer if user double-clicks in normal mode.
    if (CurrentDrawMode==0) {
        TileArray[CurrentTileX][CurrentTileY][1] = 99;
    }
}
//---------------------------------------------------------------------------



