/***************************************************
 *                                                 *
 *   NWMap v. 1.1                                  *
 *   Copyright 2001, by Jason Harris               *
 *   http://nwmap.nwnonline.net/                   *
 *   nwmap@hotmail.com                             *
 *   jharris@30doradus.org                         *
 *                                                 *
 ***************************************************/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "NWPopupFuncs.h"
#include "NWMapMain.h"
#include "TilesetFormMain.h"
#include "TileDescMain.h"
#include "DoorFormMain.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
//
//  This file contains Event handlers for the map window popup menu
//
//---------------------------------------------------------------------------

void __fastcall TForm1::DescItemClick(TObject *Sender)
{
    int result = TileDescForm->ShowModal();
    if (result == mrOk) {
        SaveMapState(true);

        // if CurrentTile is already described, remove the old entry before inserting the new one.
        int oldindex = -1;  // default to iTile not being in the list
        for (int i=0; i<TileList->Count; ++i) {
            NWTile *nwt = new NWTile((NWTile *)TileList->Items[i]);
            if (nwt->iTile == CurrentTile) {
                oldindex = i;
                TileList->Delete(oldindex);
                break;  // found the tile, stop looking
            }
        }

        // construct a NWTile object from the information in TileDescForm, and
        // Insert it into TileList
        NWTile *nwt = new NWTile(TileDescForm->nwt);

        int index = TileDescForm->LocID->Text.ToInt() - 1; // convert to zero-based count

        int StartLabel;
        if (oldindex >= 0 && oldindex < index) { // moved down in the list
            TileList->Insert(index-1, nwt); // adjust index of insertion point to account for deletion of old copy
            StartLabel = oldindex;
        } else { // inserting new description or moved up in the list
            TileList->Insert(index, nwt); // no index adjustment required
            StartLabel = index;
        }

        // Draw ID labels on the tiles that changed labels
        drawLabels(StartLabel, TileList->Count);

        // draw current label on saved tile image (so the label sticks when cursor is removed
        SaveTile->Canvas->Font->Color = clBlack;
        int xc, yc;
        if (Zoom==1.0) {
            SaveTile->Canvas->Font->Height = TILESIZE;
            xc = TILESIZE/4;
            yc = 2;
        } else if (Zoom==2.0) {
            SaveTile->Canvas->Font->Height = TILESIZE/2;
            xc = TILESIZE/3;
            yc = TILESIZE/4;
        } else {
            SaveTile->Canvas->Font->Height = TILESIZE/4;
            xc = TILESIZE/2;
            yc = TILESIZE/4;
        }
        SaveTile->Canvas->TextOut(xc, yc, (String)(index+1));

        MapChanged = true;
        SaveButton->Enabled = true;

    } else if (result == mrNo) { // remove description from the list
        bool foundTile = false;
        for (int i=0; i<TileList->Count; ++i) {
            NWTile *nwt = (NWTile *)TileList->Items[i];
            if (nwt->iTile == CurrentTile) {
                TileList->Delete(i);

                //redraw Tile without Label
                drawTile(CurrentTileX, CurrentTileY);
                drawCursor(CurrentTileX, CurrentTileY);

                //update the rest of the tiles
                drawLabels(i, TileList->Count);

                MapBox->Canvas->Draw(0,0,Map);

                foundTile = true;
                MapChanged = true;
                SaveButton->Enabled = true;

                break; // found the tile, stop looking
            }

        }
        if (!foundTile) Application->MessageBox("Tile description not found", NULL, MB_OK);
    }

}
//---------------------------------------------------------------------------


void __fastcall TForm1::DoorItemClick(TObject *Sender)
{
    int result = DoorForm->ShowModal();

    if (result==mrOk) {
        drawTile(CurrentTileX, CurrentTileY);
        SaveTile->Canvas->Draw(0,0,Tile);
        if (Zoom==1.0) {
            DoorForm->drawDoors(CurrentTileX, CurrentTileY, SaveTile->Canvas, 4);
        } else {
            DoorForm->drawDoors(CurrentTileX, CurrentTileY, SaveTile->Canvas, 2);
        }

        //Blit to screen
        MapBox->Canvas->Draw(0,0,Map);
    }

}
//---------------------------------------------------------------------------

void __fastcall TForm1::CutDescClick(TObject *Sender)
{
    SaveMapState(true);
    CopyDescClick(Form1);
    TileList->Delete(CurrentTileIndex);

    // redraw CurrentTile without Label
    drawTile(CurrentTileX, CurrentTileY);
    drawCursor(CurrentTileX, CurrentTileY);
    MapBox->Canvas->Draw(0,0,Map);

    // update all tiles below CurrentTileIndex
    drawLabels(CurrentTileIndex, TileList->Count);

    PasteDesc->Enabled = true;
    MapChanged = true;
    SaveButton->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CopyDescClick(TObject *Sender)
{
// CurrentTileIndex was defined when popup menu was opened (MouseDown)
    nwtClipboard = (NWTile *)TileList->Items[CurrentTileIndex];
    PasteDesc->Enabled = true;
}
//---------------------------------------------------------------------------


void __fastcall TForm1::PasteDescClick(TObject *Sender)
{
    SaveMapState(true);
    NWTile *nwt = new NWTile(nwtClipboard);
    nwt->iTile = CurrentTile;
    TileList->Add(nwt);

    drawLabels(TileList->Count - 1, TileList->Count);

    // draw current label on saved tile image (so the label sticks when cursor is removed)
    if (Zoom==1.0) {
        SaveTile->Canvas->Font->Height = TILESIZE;
    } else if (Zoom==2.0) {
        SaveTile->Canvas->Font->Height = TILESIZE/2;
    } else {
        SaveTile->Canvas->Font->Height = TILESIZE/4;
    }

    SaveTile->Canvas->Font->Color = clBlack;
    SaveTile->Canvas->TextOut(2, 2, (String)(TileList->Count));

    PasteDesc->Enabled = false;
    MapChanged = true;
    SaveButton->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CutRegionClick(TObject *Sender)
{
    CutTileList->Clear();
    SaveMapState(true);

    for (int ix=RectX1; ix<RectX2; ++ix) {
        for (int iy=RectY1; iy<RectY2; ++iy) {
            TileInfo *t = new TileInfo;
            t->X = ix - RectX1;
            t->Y = iy - RectY1;
            for (int i=0; i<MLAYER; ++i) {
                t->Tile[i] = TileArray[ix][iy][i];
                t->Rot[i] = RotArray[ix][iy][i];
            }

            for (int i=0; i<5; ++i) {
                t->Doors[i] = Doors[ix][iy][i];
            }

            if (Sender==CutRegion) {
                for (int i=0; i<MLAYER; ++i) {
                    TileArray[ix][iy][i] = 0;
                    RotArray[ix][iy][i] = 0;
                }
                for (int i=0; i<5; ++i) {
                    Doors[ix][iy][i] = 0;
                }
            }

            int iTile = iy*NXMAP + ix;
            bool found = false;

            for (int i=0; i<TileList->Count; ++i) { // Loop through described tiles
                NWTile *nwt = (NWTile *)TileList->Items[i];
                if (iTile == nwt->iTile) {
                    nwt->iTile = -1; //don't draw on map
                    t->iTile = i; // store value of described tile ID
                    found = true;
                    break;
                }
            }

            if (Sender==CutRegion) drawTile(ix, iy);

            if (!found) { t->iTile = -1; }
            CutTileList->Add(t);

            PasteRegion->Enabled = true;
        }
    }
    LastTileX = -1; //Don't redraw last tile
    MapBox->Canvas->Draw(0,0,Map);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PasteRegionClick(TObject *Sender)
{
    SaveMapState(true);

    for (int i=0; i<CutTileList->Count; ++i) {
        TileInfo *t = (TileInfo *)CutTileList->Items[i];
        int ix = CurrentTileX + t->X;
        int iy = CurrentTileY + t->Y;

        for (int i=0; i<MLAYER; ++i) {
            TileArray[ix][iy][i] = t->Tile[i];
            RotArray[ix][iy][i] = t->Rot[i];
        }
        for (int i=0; i<5; ++i) {
            Doors[ix][iy][i] = t->Doors[i];
        }

        if (t->iTile >= 0) {
            NWTile *nwt = (NWTile *)TileList->Items[t->iTile]; //t->iTile is the described tile's ID
            nwt->iTile = iy*NXMAP + ix; //set new position in map
        }
        drawTile(ix,iy);
    }

    LastTileX = -1; //Don't redraw last tile
    MapBox->Canvas->Draw(0,0,Map);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::RotateClick(TObject *Sender)
{
    SaveMapState(true);
    if (Sender==LRotate) {
        --RotArray[CurrentTileX][CurrentTileY][CurrentLayer];
        if (RotArray[CurrentTileX][CurrentTileY][CurrentLayer] < 0) RotArray[CurrentTileX][CurrentTileY][CurrentLayer] = 3;
    } else if (Sender==RRotate) {
        ++RotArray[CurrentTileX][CurrentTileY][CurrentLayer];
        if (RotArray[CurrentTileX][CurrentTileY][CurrentLayer] >3) RotArray[CurrentTileX][CurrentTileY][CurrentLayer] = 0;
    }

    drawCursor(CurrentTileX, CurrentTileY);
    MapBox->Canvas->Draw(0,0,Map);

    MapChanged = true;
    SaveButton->Enabled = true;
}
