/***************************************************
 *                                                 *
 *   NWMap v. 1.1                                  *
 *   Copyright 2001, by Jason Harris               *
 *   http://nwmap.nwnonline.net/                   *
 *   nwmap@hotmail.com                             *
 *   jharris@30doradus.org                         *
 *                                                 *
 ***************************************************/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <fstream.h>
#include "TilesetFormMain.h"
#include "NWConst.h"
#include "NWMapMain.h"
#include "DoorFormMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTilesetForm *TilesetForm;
//---------------------------------------------------------------------------

__fastcall TTilesetForm::TTilesetForm(TComponent* Owner)
    : TForm(Owner)
{
    SizeFactor = 2;
    for (int iLayer=0; iLayer<TLAYER; ++iLayer) {
        TileBox[iLayer] = new TPaintBox( PageControl1->Pages[iLayer] );
        TileBox[iLayer]->Parent = PageControl1->Pages[iLayer];
        TileBox[iLayer]->Top = 0;
        TileBox[iLayer]->Left = 40;
        TileBox[iLayer]->Width = 200*SizeFactor;
        TileBox[iLayer]->Height = 200*SizeFactor;
        TileBox[iLayer]->ShowHint = true;
        TileBox[iLayer]->OnMouseMove = TileBoxMouseMove;
        TileBox[iLayer]->OnMouseUp = TileBoxMouseUp;
    }
}
//---------------------------------------------------------------------------

void __fastcall TTilesetForm::FormCreate(TObject *Sender)
{
    Left = Form1->Left + Form1->Width + 20;
    Top = Form1->Top;
    Height = 500;
    CurrentLayer = 0;

    BorderIcons = BorderIcons - (TBorderIcons() << biMaximize);
    PageControl1->ActivePage = Tab0;

    Tile = new Graphics::TBitmap();
    Tile->Width = TILESIZE;
    Tile->Height = TILESIZE;
    Tile2 = new Graphics::TBitmap();
    Tile2->Width = TILESIZE;
    Tile2->Height = TILESIZE;

    Panel[0][1] = new TPanel( Tab1 );
    Panel[0][1]->Parent = Tab1;
    Panel[0][2] = new TPanel( Tab1 );
    Panel[0][2]->Parent = Tab1;
    Panel[1][1] = new TPanel( Tab2 );
    Panel[1][1]->Parent = Tab2;
    Panel[1][2] = new TPanel( Tab2 );
    Panel[1][2]->Parent = Tab2;
    Panel[2][1] = new TPanel( Tab3 );
    Panel[2][1]->Parent = Tab3;
    Panel[2][2] = new TPanel( Tab3 );
    Panel[2][2]->Parent = Tab3;
    Panel[3][1] = new TPanel( Tab4 );
    Panel[3][1]->Parent = Tab4;
    Panel[3][2] = new TPanel( Tab4 );
    Panel[3][2]->Parent = Tab4;

    for (int iLayer=0; iLayer<TLAYER; ++iLayer) {
        Tileset[iLayer] = new Graphics::TBitmap();
        Tileset[iLayer]->Width = TileBox[iLayer]->Width;
        Tileset[iLayer]->Height = TileBox[iLayer]->Height;
        TileNames[iLayer] = new TStringList();
        GroupNames[iLayer] = new TStringList();

        SelectedTile[iLayer] = 0;

        for (int irot=0; irot<4; ++irot) {
            Tilebmp[irot][iLayer] = new Graphics::TBitmap();
            Tilebmp[irot][iLayer]->Width = TILESIZE*NXTILES + 2;
            Tilebmp[irot][iLayer]->Height = TILESIZE*NYTILES + 2;
            Tilebmp[irot][iLayer]->LoadFromFile( "tilesets\\outdoor_bases.bmp" );
        }

        if (iLayer>0) {
            Panel[iLayer-1][1]->Left = 0;
            Panel[iLayer-1][1]->Top = 0;
            Panel[iLayer-1][1]->Width = 38;
            Panel[iLayer-1][1]->Height = 200;
            Panel[iLayer-1][2]->Left = 0;
            Panel[iLayer-1][2]->Top = 200;
            Panel[iLayer-1][2]->Width = 38;
            Panel[iLayer-1][2]->Height = 200;

            for (int i=0; i<8; ++i) {
                GButton[iLayer-1][i] = new TSpeedButton( this );
                GButton[iLayer-1][i]->Parent = Panel[iLayer-1][1];
                GButton[iLayer-1][i]->Left = 8;
                GButton[iLayer-1][i]->Top = 4 + i*24;
                GButton[iLayer-1][i]->Width = 22;
                GButton[iLayer-1][i]->Height = 22;
                GButton[iLayer-1][i]->OnClick = ModeButtonClick;
                GButton[iLayer-1][i]->Visible = false;
                GButton[iLayer-1][i+8] = new TSpeedButton( this );
                GButton[iLayer-1][i+8]->Parent = Panel[iLayer-1][2];
                GButton[iLayer-1][i+8]->Left = 8;
                GButton[iLayer-1][i+8]->Top = 4 + i*24;
                GButton[iLayer-1][i+8]->Width = 22;
                GButton[iLayer-1][i+8]->Height = 22;
                GButton[iLayer-1][i+8]->Visible = false;
                GButton[iLayer-1][i+8]->OnClick = ModeButtonClick;
            }
        }
    }

    TileRect = Rect(0,0,TILESIZE,TILESIZE);

    LoadTileset(0); //Start with Default Tileset

    //Initialize Tileset offset coords
    int i = 0;
    for (int iy=0; iy<NYTILES; ++iy) {
        for (int ix=0; ix<NXTILES; ++ix) {
            XTileset[i] = ix;
            YTileset[i] = iy;
            ++i;
        }
    }

    drawTiles( CurrentLayer );
    Form1->drawMap();
}
//---------------------------------------------------------------------------


void __fastcall TTilesetForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
    if (Sender == TilesetForm) {
        Action = caHide;
    } else {
        for (int iLayer=0; iLayer<TLAYER; ++iLayer) {
            delete Tileset[iLayer];
            delete TileNames[iLayer];
            delete TileBox[iLayer];
            delete GroupNames[iLayer];

            for (int i=0; i<4; ++i) {
                delete Tilebmp[i][iLayer];
            }

            if (iLayer>0) {
                for(int i=0; i<NGROUP; ++i) {
                    delete GButton[iLayer-1][i];
                }
                delete Panel[iLayer-1][1];
                delete Panel[iLayer-1][2];
            }
        }

        delete Tile;
    }
}
//---------------------------------------------------------------------------

void TTilesetForm::LoadTileset(int i) {
    SetCurrentDir( Form1->NWMDir );

    for (int iLayer=0; iLayer< TLAYER; ++iLayer) {
        for (int irot=0; irot<4; ++irot) {
            delete Tilebmp[irot][iLayer];
            Tilebmp[irot][iLayer] = new Graphics::TBitmap;
            Tilebmp[irot][iLayer]->Width = TILESIZE*NXTILES + 2;
            Tilebmp[irot][iLayer]->Height = TILESIZE*NYTILES + 2;
        }
    }

    TileTitle->Text = Form1->TilesetNames->Strings[i];

    for (int iLayer=0; iLayer<TLAYER; ++iLayer) {
        if (Form1->TilesetFiles[iLayer]->Strings[i] != "null") {
            if (Tilebmp[0][iLayer]==NULL) {
                Tilebmp[0][iLayer]->Width = TILESIZE*NXTILES + 2;
                Tilebmp[0][iLayer]->Height = TILESIZE*NYTILES + 2;
            }

            Tilebmp[0][iLayer]->LoadFromFile(Form1->TilesetFiles[iLayer]->Strings[i]);
            numLayer = iLayer+1;
            PageControl1->Pages[iLayer]->TabVisible = true;
            SelectedTile[iLayer] = 0;
        } else {
            Tilebmp[0][iLayer] = NULL;
            PageControl1->Pages[iLayer]->TabVisible = false;
        }
    }

    BlankTile = 0;
    Form1->nmLayer = numLayer + 1; //set number of map layers

    // Rotated Tilesets start out blank
    for (int irot=1; irot<4; ++irot) {
        for (int iLayer=0; iLayer<numLayer; ++iLayer) {
            Tilebmp[irot][iLayer]->Canvas->Brush->Color = clBlack;
            Tilebmp[irot][iLayer]->Canvas->FillRect(Rect(0,0,Tilebmp[irot][iLayer]->Width,Tilebmp[irot][iLayer]->Height));
        }
    }

    for (int iLayer=0; iLayer<numLayer; ++iLayer) {
        if (Tilebmp[0][iLayer] != NULL) {
            TRect SetRect = Rect(0, 0, Tileset[iLayer]->Width, Tileset[iLayer]->Height);
            Tileset[iLayer]->Canvas->StretchDraw(SetRect, Tilebmp[0][iLayer]);
        } else {
            Tileset[iLayer] = NULL;
        }
    }
    //Initialize Tileset rotation states
    int iTile = 0;
    for (int iy=0; iy<NYTILES; ++iy) {
        for (int ix=0; ix<NXTILES; ++ix) {
            for (int irot=1; irot<4; ++irot) {
                for (int iLayer=0; iLayer<numLayer; ++iLayer) {
                    SeenIt[irot][iLayer][iTile] = false;
//                    rotateTile( iTile, irot, iLayer);
                }
            }

            for (int iLayer=0; iLayer<numLayer; ++iLayer) {
                SeenIt[0][iLayer][iTile] = true;
                Rotation[iTile][iLayer] = 0;
            }

            ++iTile;
        }
    }

    //Define Tile groups:
    iset = i;
    for (int iLayer=0; iLayer<numLayer; ++iLayer) {
        Form1->DrawModes[iLayer]->Clear();
        Form1->DrawModes[iLayer]->Add("Normal Mode");
        Form1->DrawModes[iLayer]->Add("Selection Mode");
        if (iLayer>0) {
            setGroups( iLayer );
            Form1->DrawModes[iLayer]->AddStrings((TStrings *)GroupNames[iLayer]);
        }
    }

    Form1->CurrentDrawMode = 0;
    Form1->SnakeMode = false;
    Form1->Label1->Caption = Form1->DrawModes[CurrentLayer]->Strings[Form1->CurrentDrawMode];

    //Attempt to load tile tooltips for this tileset:
    for (int iLayer=0; iLayer<numLayer; ++iLayer) {
        FILE *TipFile;
        String TipFileName = Form1->TilesetFiles[iLayer]->Strings[i].SubString(0,
                            Form1->TilesetFiles[iLayer]->Strings[i].Pos(".bmp")) +
                            "tip";
        if (TileNames[iLayer]->Count > 0) { TileNames[iLayer]->Clear(); }

        if ((TipFile = fopen(TipFileName.c_str(), "rt")) != NULL) {
            char buff[80];
            char *check;

            while (1) {
                check = fgets(buff, sizeof(buff), TipFile);
                if (check==NULL) break;
                check = strchr(buff, '\n'); //find the \n character...
                *check = '\0';              //...and eat it.
                TileNames[iLayer]->Add((String)buff);
            }
        }
        fclose(TipFile);

        drawTiles(iLayer);
    }
}
//---------------------------------------------------------------------------

void TTilesetForm::getTile(Graphics::TBitmap *Tile, int iTile, int nrot, int iLayer) {
    HDC tileDC = Tile->Canvas->Handle;
    SetStretchBltMode( tileDC, COLORONCOLOR );
    if (SeenIt[nrot][iLayer][iTile]==false) {
        rotateTile( iTile, nrot, iLayer);
    }

    int ix = XTileset[iTile];
    int iy = YTileset[iTile];

    SelectRect = Rect(ix*TILESIZE+1, iy*TILESIZE+1, (ix+1)*TILESIZE+1, (iy+1)*TILESIZE+1);
    Tile->Canvas->CopyMode = cmSrcCopy;
    Tile->Canvas->CopyRect(TileRect, Tilebmp[nrot][iLayer]->Canvas, SelectRect);
}
//---------------------------------------------------------------------------

void TTilesetForm::drawTile(int iTile, int nrot, int iLayer) {
    if (iTile>=NTILES || nrot >=4 || iLayer>=5 ||
        iTile<0 || nrot <0 || iLayer<0) {
        Application->MessageBox( "Illegal Tile Request", NULL, MB_OK);
    }


    switch (iLayer) {
        case 0: //base layer, draw on black background
            Tile->Canvas->Brush->Color = clBlack;
            Tile->Canvas->FillRect( TileRect );
            getTile( Tile2, iTile, nrot, iLayer );
            Form1->TransDraw( Tile, Tile2 );
            break;
        default: //draw tile on default base layer
            getTile( Tile, 0, 0, 0 );
            getTile( Tile2, iTile, nrot, iLayer );
            Form1->TransDraw( Tile, Tile2 );
            break;
    }

    SetRect = Rect(XTileset[iTile]*SizeFactor*ROOTSIZE+1, YTileset[iTile]*SizeFactor*ROOTSIZE+1,
                   (XTileset[iTile]+1)*SizeFactor*ROOTSIZE+1, (YTileset[iTile]+1)*SizeFactor*ROOTSIZE+1);
    Tileset[iLayer]->Canvas->StretchDraw(SetRect, Tile);
}
//---------------------------------------------------------------------------

void TTilesetForm::rotateTile(int iTile, int nrot, int iLayer) {
    if (SeenIt[nrot][iLayer][iTile]==false) {
        if (nrot >=4) {
            Application->MessageBox( "Bad rotation value!", NULL, MB_OK);
        }
        HDC tileDC = Tile->Canvas->Handle;
        SetStretchBltMode( tileDC, COLORONCOLOR );

        // Load unrotated tile bitmap into Tile
        int xx = XTileset[iTile];
        int yy = YTileset[iTile];
        TRect SelectRect = Rect(xx*TILESIZE+1, yy*TILESIZE+1, (xx+1)*TILESIZE+1, (yy+1)*TILESIZE+1);
        Tile->Canvas->CopyMode = cmSrcCopy;
        Tile->Canvas->CopyRect(TileRect, Tilebmp[0][iLayer]->Canvas, SelectRect);

        TColor tPix[TILESIZE][TILESIZE];

        for (int ix=0; ix<TILESIZE; ++ix) {
            for (int iy=0; iy<TILESIZE; ++iy) {
                switch (nrot) {
                    case 1:
                        tPix[ix][iy] = Tile->Canvas->Pixels[iy][TILESIZE-ix-1];
                        break;
                    case 2:
                        tPix[ix][iy] = Tile->Canvas->Pixels[TILESIZE-ix-1][TILESIZE-iy-1];
                        break;
                    case 3:
                        tPix[ix][iy] = Tile->Canvas->Pixels[TILESIZE-iy-1][ix];
                        break;
                    default:  //hopefully, will never see this!
                        Application->MessageBox("Error: Illegal rotation value!", NULL, MB_OK);
                }
            }
        }

        for (int jx=0; jx<TILESIZE; ++jx) {
            for (int jy=0; jy<TILESIZE; ++jy) {
                Tilebmp[nrot][iLayer]->Canvas->Pixels[xx*TILESIZE+jx+1][yy*TILESIZE+jy+1] = tPix[jx][jy];
            }
        }

        SeenIt[nrot][iLayer][iTile] = true;
        drawTile( iTile, nrot, iLayer );
    }
}

//---------------------------------------------------------------------------

void TTilesetForm::drawTiles(int iLayer)
{
    if (Tileset[iLayer] != NULL) {
        for (int i=0; i<NTILES; ++i) {
            drawTile(i, Rotation[i][iLayer], iLayer);
        }
        TColor c = clMaroon;
        if (Form1->CurrentDrawMode > 0) c = clGreen;
        drawCursor(SelectedTile[iLayer], iLayer, c);
        TileBox[iLayer]->Canvas->Draw(0,0,Tileset[iLayer]);
    }
}
//---------------------------------------------------------------------------

void TTilesetForm::drawCursor(int iTile, int iLayer, TColor col)
{
    int xt = XTileset[iTile];
    int yt = YTileset[iTile];

    Tileset[iLayer]->Canvas->Pen->Color = col;
    Tileset[iLayer]->Canvas->Pen->Width = 1;
    Tileset[iLayer]->Canvas->MoveTo(xt*SizeFactor*ROOTSIZE + 2, yt*SizeFactor*ROOTSIZE + 2);
    Tileset[iLayer]->Canvas->LineTo((xt+1)*SizeFactor*ROOTSIZE - 2, yt*SizeFactor*ROOTSIZE + 2);
    Tileset[iLayer]->Canvas->LineTo((xt+1)*SizeFactor*ROOTSIZE - 2, (yt+1)*SizeFactor*ROOTSIZE - 2);
    Tileset[iLayer]->Canvas->LineTo(xt*SizeFactor*ROOTSIZE + 2, (yt+1)*SizeFactor*ROOTSIZE - 2);
    Tileset[iLayer]->Canvas->LineTo(xt*SizeFactor*ROOTSIZE + 2, yt*SizeFactor*ROOTSIZE + 2);
}

//---------------------------------------------------------------------------

void __fastcall TTilesetForm::FormPaint(TObject *Sender)
{
    TileBox[CurrentLayer]->Canvas->Draw(0,0,Tileset[CurrentLayer]);
}
//---------------------------------------------------------------------------

void __fastcall TTilesetForm::TileBoxMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    //  Select the Tile that was clicked

    int ix = int(X/ROOTSIZE/SizeFactor);
    int iy = int(Y/ROOTSIZE/SizeFactor);

    if (ix!=oldx || iy!=oldy) {
        drawTile(SelectedTile[CurrentLayer], Rotation[SelectedTile[CurrentLayer]][CurrentLayer], CurrentLayer);
    }

    oldx = ix;  oldy = iy;

    SelectedTile[CurrentLayer] = iy*NXTILES + ix;
    drawTile(SelectedTile[CurrentLayer], Rotation[SelectedTile[CurrentLayer]][CurrentLayer], CurrentLayer);
    TColor color = clMaroon;
    Form1->CurrentDrawMode = 0;
    Form1->SnakeMode = false;
    Form1->Label1->Caption = Form1->DrawModes[CurrentLayer]->Strings[Form1->CurrentDrawMode];

    Form1->CurrentLayer = CurrentLayer + 1; //set current map layer to current tileset layer + 1
    if ( CurrentLayer==0 && ix==0 ) { //...unless it's the base layer, then map layer depends on selected tile.
        Form1->CurrentLayer = 0;
        //If CheckBox is checked, set CurrentDrawMode to -1.
        switch (iy) {
            case 0:
                if ( CheckBox1->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 1:
                if ( CheckBox2->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 2:
                if ( CheckBox3->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 3:
                if ( CheckBox4->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 4:
                if ( CheckBox5->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 5:
                if ( CheckBox6->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 6:
                if ( CheckBox7->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 7:
                if ( CheckBox8->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 8:
                if ( CheckBox9->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
            case 9:
                if ( CheckBox10->Checked ) {
                    Form1->CurrentDrawMode = -1;
                    Form1->Label1->Caption = "Draw Terrain";
                }
                break;
        }

    }

    if (Form1->RegionSelected) {
        Form1->RegionSelected = false;
        Form1->CutRegion->Enabled = false;
        Form1->CopyRegion->Enabled = false;
        Form1->drawMap();
    }

    if (Button == mbLeft) {
        drawCursor(SelectedTile[CurrentLayer], CurrentLayer, color);
    } else if (Button == mbRight) {
        bool breakout = false;

        int i;
        for (i=0; i<NGROUP; ++i) { //Did you right-click on a grouped tile?
            for (int iTile=0; iTile<4; ++iTile) {
                if (SelectedTile[CurrentLayer] == BTile[CurrentLayer-1][i][iTile]) {
                    Form1->SnakeMode = false;
                    breakout = true;
                    Form1->CurrentDrawMode = i + 2;
                    break;
                } else if (SelectedTile[CurrentLayer] == STile[CurrentLayer-1][i][iTile]) {
                    Form1->SnakeMode = true;
                    breakout = true;
                    Form1->CurrentDrawMode = i + 10;
                    break;
                }
            }
            if (breakout) break;
        }

        if (breakout) { //rt-click grouped tile, so enter the appropriate mode
            color = clGreen;
            Form1->Label1->Caption = Form1->DrawModes[CurrentLayer]->Strings[Form1->CurrentDrawMode];
        }

        drawCursor(SelectedTile[CurrentLayer], CurrentLayer, color);
    }

    TileBox[CurrentLayer]->Canvas->Draw(0,0,Tileset[CurrentLayer]);
}

//---------------------------------------------------------------------------
void __fastcall TTilesetForm::TileBoxMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
    int ix = int(X/ROOTSIZE/SizeFactor);
    int iy = int(Y/ROOTSIZE/SizeFactor);

    if (ix != px || iy != py) {
        px = ix; py = iy;
        int i = iy*NXTILES + ix;

        if (i < TileNames[CurrentLayer]->Count) {
            TileBox[CurrentLayer]->Hint = TileNames[CurrentLayer]->Strings[i];
        } else {
            TileBox[CurrentLayer]->Hint = "No Description";
        }
    }
}

//---------------------------------------------------------------------------
void __fastcall TTilesetForm::FormKeyPress(TObject *Sender, char &Key)
{
    Form1->FormKeyPress(Sender, Key);
}
//---------------------------------------------------------------------------

void __fastcall TTilesetForm::FormShow(TObject *Sender)
{
    int size = 200*SizeFactor;

    for (int i=0; i<TLAYER; ++i) {
        TileBox[i]->Width = size;
        TileBox[i]->Height = size;
        Tileset[i]->Width = size;
        Tileset[i]->Height = size;
    }

    Width = size + 56;
    if (SizeFactor==1) Width += 40; //account for double-parked button panels
    Height = size + 80;
    drawTiles(CurrentLayer);
}
//---------------------------------------------------------------------------

void TTilesetForm::setGroups( int iLayer )
{
    //default:  no groups defined
    GroupNames[iLayer]->Clear();

    for(int i=0; i<NGROUP; ++i) {
        GButton[iLayer-1][i]->Visible = false;
    }

    FILE *GroupFile;
    char *check;
    String gname, bmpname[16];
    int nb = 0;
    int ns = 0;
    int ntot = 0;

    String GrpFileName = Form1->TilesetFiles[iLayer]->Strings[iset].SubString(0,
                        Form1->TilesetFiles[iLayer]->Strings[iset].Pos(".bmp")) +
                        "grp";
    TStringList *Names = new TStringList();

    if ((GroupFile = fopen( GrpFileName.c_str(), "rt" )) != NULL) {
        char buff[140];

        while (1) {
            check = fgets(buff, sizeof(buff), GroupFile);
            if (check==NULL) break;
            check = strchr(buff, '\n'); //find the \n character...
            *check = '\0';              //...and eat it.
            String s = String(buff).Trim();
            String sub = s.SubString( s.Pos(":")+1, s.Length() );

            if (buff[0]=='B' && ntot<16) { //This line defines a BoxMode
                String nums = sub.SubString( 0, sub.Pos(":")-1 ).Trim();
                String names = sub.SubString( sub.Pos(":")+1, sub.Length() ).Trim();

                gname = names.SubString( 0, names.Pos(":")-1 ).Trim();
                bmpname[ntot] = names.SubString( names.Pos(":")+1, names.Length() ).Trim();
                gType[iLayer-1][ntot] = nb+1; // >0 means Box mode

                sscanf(nums.c_str(), "%d %d %d %d %d %d %d %d",
                                    &BTile[iLayer-1][nb][0], &BRot[iLayer-1][nb][0],
                                    &BTile[iLayer-1][nb][1], &BRot[iLayer-1][nb][1],
                                    &BTile[iLayer-1][nb][2], &BRot[iLayer-1][nb][2],
                                    &BTile[iLayer-1][nb][3], &BRot[iLayer-1][nb][3] );
                Names->Add( gname );
                ++nb;
                ++ntot;

            } else if (buff[0]=='S' && ntot<16) { //This line defines a SnakeMode
                String nums1 = sub.SubString( 0, sub.Pos(":")-1 ).Trim();
                String rem1 = sub.SubString( sub.Pos(":")+1, sub.Length() ).Trim();
                String nums2 = rem1.SubString( 0, rem1.Pos(":")-1 ).Trim();
                String rem2 = rem1.SubString( rem1.Pos(":")+1, rem1.Length() ).Trim();
                String nums3 = rem2.SubString( 0, rem2.Pos(":")-1 ).Trim();
                String names = rem2.SubString( rem2.Pos(":")+1, rem2.Length() ).Trim();
                gname = names.SubString( 0, names.Pos(":")-1 ).Trim();
                bmpname[ntot] = names.SubString( names.Pos(":")+1, names.Length() ).Trim();
                gType[iLayer-1][ntot] = -1*(ns+1); // <0 means Snake mode

                sscanf( nums1.c_str(), "%d %d %d %d %d %d %d %d",
                        &STile[iLayer-1][ns][0], &SRot[iLayer-1][ns][0], &STile[iLayer-1][ns][1], &SRot[iLayer-1][ns][1],
                        &STile[iLayer-1][ns][2], &SRot[iLayer-1][ns][2], &STile[iLayer-1][ns][3], &SRot[iLayer-1][ns][3] );

                sscanf( nums2.c_str(), "%d %d %d %d %d %d %d %d %d %d %d %d",
                        &SCrossTile[iLayer-1][ns][0], &SCReplaceTile[iLayer-1][ns][0], &SCReplaceRot[iLayer-1][ns][0],
                        &SCrossTile[iLayer-1][ns][1], &SCReplaceTile[iLayer-1][ns][1], &SCReplaceRot[iLayer-1][ns][1],
                        &SCrossTile[iLayer-1][ns][2], &SCReplaceTile[iLayer-1][ns][2], &SCReplaceRot[iLayer-1][ns][2],
                        &SCrossTile[iLayer-1][ns][3], &SCReplaceTile[iLayer-1][ns][3], &SCReplaceRot[iLayer-1][ns][3] );

                sscanf( nums3.c_str(), "%d %d %d %d %d %d %d %d %d %d %d %d",
                        &SEndFlag[iLayer-1][ns][0], &SEndTile[iLayer-1][ns][0], &SEndRot[iLayer-1][ns][0],
                        &SEndFlag[iLayer-1][ns][1], &SEndTile[iLayer-1][ns][1], &SEndRot[iLayer-1][ns][1],
                        &SEndFlag[iLayer-1][ns][2], &SEndTile[iLayer-1][ns][2], &SEndRot[iLayer-1][ns][2],
                        &SEndFlag[iLayer-1][ns][3], &SEndTile[iLayer-1][ns][3], &SEndRot[iLayer-1][ns][3] );

                Names->Add( gname );
                ++ns;
                ++ntot;
            }
        }
    }
    fclose(GroupFile);

    for (int i=0; i<ntot; ++i) {
        GButton[iLayer-1][i]->Visible = true;
        GButton[iLayer-1][i]->Glyph->LoadFromFile( bmpname[i].c_str() );
        GButton[iLayer-1][i]->Hint = Names->Strings[i];
    }

    for (int i=ntot; i<NGROUP; ++i) {
        Names->Add( "junk" ); //will never see this; need it as a placeholder
    }

    GroupNames[iLayer]->AddStrings( Names );
    delete Names;
}


void __fastcall TTilesetForm::PageControl1Change(TObject *Sender)
{

    if (PageControl1->ActivePage->Name=="Tab0") {
        CurrentLayer = 0;
        Form1->CurrentLayer = 0;
    } else if (PageControl1->ActivePage->Name=="Tab1") {
        CurrentLayer = 1;
        Form1->CurrentLayer = 2;
    } else if (PageControl1->ActivePage->Name=="Tab2") {
        CurrentLayer = 2;
        Form1->CurrentLayer = 3;
    } else if (PageControl1->ActivePage->Name=="Tab3") {
        CurrentLayer = 3;
        Form1->CurrentLayer = 4;
    } else if (PageControl1->ActivePage->Name=="Tab4") {
        CurrentLayer = 4;
        Form1->CurrentLayer = 5;
    }

    drawTiles( CurrentLayer );
    if ( Form1->OneLayerMode ) Form1->drawMap();
}
//---------------------------------------------------------------------------



void __fastcall TTilesetForm::ModeButtonClick(TObject *Sender)
{
    drawTile( SelectedTile[CurrentLayer],  Rotation[SelectedTile[CurrentLayer]][CurrentLayer], CurrentLayer );
    Form1->SnakeMode = false;

    if (Sender==GButton[0][0] || Sender==GButton[1][0] || Sender==GButton[2][0] || Sender==GButton[3][0]) {
        Form1->CurrentDrawMode = 2;
    } else if (Sender==GButton[0][1] || Sender==GButton[1][1] || Sender==GButton[2][1] || Sender==GButton[3][1]) {
        Form1->CurrentDrawMode = 3;
    } else if (Sender==GButton[0][2] || Sender==GButton[1][2] || Sender==GButton[2][2] || Sender==GButton[3][2]) {
        Form1->CurrentDrawMode = 4;
    } else if (Sender==GButton[0][3] || Sender==GButton[1][3] || Sender==GButton[2][3] || Sender==GButton[3][3]) {
        Form1->CurrentDrawMode = 5;
    } else if (Sender==GButton[0][4] || Sender==GButton[1][4] || Sender==GButton[2][4] || Sender==GButton[3][4]) {
        Form1->CurrentDrawMode = 6;
    } else if (Sender==GButton[0][5] || Sender==GButton[1][5] || Sender==GButton[2][5] || Sender==GButton[3][5]) {
        Form1->CurrentDrawMode = 7;
    } else if (Sender==GButton[0][6] || Sender==GButton[1][6] || Sender==GButton[2][6] || Sender==GButton[3][6]) {
        Form1->CurrentDrawMode = 8;
    } else if (Sender==GButton[0][7] || Sender==GButton[1][7] || Sender==GButton[2][7] || Sender==GButton[3][7]) {
        Form1->CurrentDrawMode = 9;
    } else if (Sender==GButton[0][8] || Sender==GButton[1][8] || Sender==GButton[2][8] || Sender==GButton[3][8]) {
        Form1->CurrentDrawMode = 10;
    } else if (Sender==GButton[0][9] || Sender==GButton[1][9] || Sender==GButton[2][9] || Sender==GButton[3][9]) {
        Form1->CurrentDrawMode = 11;
    } else if (Sender==GButton[0][10] || Sender==GButton[1][10] || Sender==GButton[2][10] || Sender==GButton[3][10]) {
        Form1->CurrentDrawMode = 12;
    } else if (Sender==GButton[0][11] || Sender==GButton[1][11] || Sender==GButton[2][11] || Sender==GButton[3][11]) {
        Form1->CurrentDrawMode = 13;
    } else if (Sender==GButton[0][12] || Sender==GButton[1][12] || Sender==GButton[2][12] || Sender==GButton[3][12]) {
        Form1->CurrentDrawMode = 14;
    } else if (Sender==GButton[0][13] || Sender==GButton[1][13] || Sender==GButton[2][13] || Sender==GButton[3][13]) {
        Form1->CurrentDrawMode = 15;
    } else if (Sender==GButton[0][14] || Sender==GButton[1][14] || Sender==GButton[2][14] || Sender==GButton[3][14]) {
        Form1->CurrentDrawMode = 16;
    } else if (Sender==GButton[0][15] || Sender==GButton[1][15] || Sender==GButton[2][15] || Sender==GButton[3][15]) {
        Form1->CurrentDrawMode = 17;
    }

    if (gType[CurrentLayer-1][Form1->CurrentDrawMode-2] < 0) {
        int imode = -1*gType[CurrentLayer-1][Form1->CurrentDrawMode-2] - 1; // make index positive, starting at zero.
        Form1->SnakeMode = true;
        SelectedTile[CurrentLayer] = STile[CurrentLayer-1][imode][0];
    } else {
        int imode = gType[CurrentLayer-1][Form1->CurrentDrawMode-2] - 1; // make index positive, starting at zero.
        SelectedTile[CurrentLayer] = BTile[CurrentLayer-1][imode][0];
    }

    Form1->Label1->Caption = Form1->DrawModes[CurrentLayer]->Strings[Form1->CurrentDrawMode];

    Form1->RegionSelected = false;
    Form1->CutRegion->Enabled = false;
    Form1->CopyRegion->Enabled = false;
    //Form1->drawMap();

    drawCursor( SelectedTile[CurrentLayer], CurrentLayer, clGreen);
    TileBox[CurrentLayer]->Canvas->Draw( 0, 0, Tileset[CurrentLayer] );

}
//---------------------------------------------------------------------------


void __fastcall TTilesetForm::oneLayerBoxClick(TObject *Sender)
{
    Form1->OneLayerMode = oneLayerBox->Checked;
    Form1->drawMap();
}
//---------------------------------------------------------------------------

void __fastcall TTilesetForm::OnTopBoxClick(TObject *Sender)
{
    if( OnTopBox->Checked ) {
        FormStyle	= fsStayOnTop;
    } else {
        FormStyle	= fsNormal;
    }
}
//---------------------------------------------------------------------------


void __fastcall TTilesetForm::CheckBoxClick(TObject *Sender)
{
    if ( ((TCheckBox *)Sender)->Checked ) {
        Form1->CurrentDrawMode = -1;
        Form1->Label1->Caption = "Draw Terrain";
    } else {
        Form1->CurrentDrawMode = 0;
        Form1->Label1->Caption = "Normal Mode";
    }

    int ix=0;
    int iy;

    if ( Sender==CheckBox1 ) { iy = 0; }
    else if ( Sender==CheckBox2 ) { iy = 1; }
    else if ( Sender==CheckBox3 ) { iy = 2; }
    else if ( Sender==CheckBox4 ) { iy = 3; }
    else if ( Sender==CheckBox5 ) { iy = 4; }
    else if ( Sender==CheckBox6 ) { iy = 5; }
    else if ( Sender==CheckBox7 ) { iy = 6; }
    else if ( Sender==CheckBox8 ) { iy = 7; }
    else if ( Sender==CheckBox9 ) { iy = 8; }
    else if ( Sender==CheckBox10 ) { iy = 9; }

    //draw previous tile to erase selection box
    drawTile(SelectedTile[CurrentLayer], Rotation[SelectedTile[CurrentLayer]][CurrentLayer], CurrentLayer);

    oldx = ix;  oldy = iy;

    SelectedTile[CurrentLayer] = iy*NXTILES + ix;
    drawTile(SelectedTile[CurrentLayer], Rotation[SelectedTile[CurrentLayer]][CurrentLayer], CurrentLayer);

    TColor color = clMaroon;
    drawCursor(SelectedTile[CurrentLayer], CurrentLayer, color);
}
//---------------------------------------------------------------------------


